# MVPLN-VAR-monthly-dynamics.R
#
# Computes the dynamic multipliers for MVPLN-VAR models.
#
#
# Patrick T. Brandt
#
# 20100319 : Initial version
# 20100426 : Updated to use new variance computations.
# 20100712 : Updated code to work for VAR(1) models selected by DIC
#            criteria from among 15 specifications.
# 20101104 : Revised for shorter IRF computations
# 20101214 : Added the computation of the correlations for the RF of
#            the BaP-VAR in the musigma() function, renamed now to
#            bapvarmoments().
# 20110103 : Updated covariances for the BaP-VAR moments computation
# 20110525 : Version for the superpower rivalry example

library(coda)

# vech function for efficiently sorting symmetric matrices (like Sigma)

vech <- function (x)
{
    x <- as.matrix(x)
    if (dim(x)[1] != dim(x)[2]) {
        stop("Non-square matrix passed to vech().\n")
    }
    output <- x[lower.tri(x, diag = TRUE)]
    dim(output) <- NULL
    return(output)
}

# xpnd function for reconstructing symmetric matrices from vech

xpnd <- function (x, nrow = NULL)
{
    dim(x) <- NULL
    if (is.null(nrow))
        nrow <- (-1 + sqrt(1 + 8 * length(x)))/2
    output <- matrix(0, nrow, nrow)
    output[lower.tri(output, diag = TRUE)] <- x
    hold <- output
    hold[upper.tri(hold, diag = TRUE)] <- 0
    output <- output + t(hold)
    return(output)
}

# bapvarmoments() -- computes the mean and variance for the
# measurement equation of the model.  As an aside, it also computes
# the correlation matrix of the errors.
bapvarmoments <- function(post, x, y, m, p)
{
    ncoef <- (m^2)*p
    ncoef2 <- ncoef + 2*m^2 + 1
    ncoef3 <- length(post)

    # Need to reorder the elements for the VAR(p) matrix here.
    # Depends on how jags returns the A matrices.
    A <- matrix(post[1:ncoef], m*p, m)

    # Version with covariates other than an intercept
    exog.coef <- matrix(post[ncoef2:ncoef3], nrow=m)
    eb <- tcrossprod(cbind(rep(1,nrow(x)), x), exog.coef)

    # Version with no covariates
    #eb <- exp(matrix(post[ncoef2:ncoef3], m, 1))
    DD <- matrix(post[(ncoef + m^2 + 1):(ncoef2-1)], m, m)
    bt <- exp(0.5*diag(DD))
    #bteb <- as.vector(bt*eb)
    bteb <- matrix(bt, nrow=nrow(x), m, byrow=TRUE)*eb
    diagbt <- array(apply(bteb, 1, diag), c(m,m,nrow(bteb))) #diag(bteb)

    # Compute the relevant covariance matrices on the scales of the
    # DVs.
    Sigma1 <- matrix(0, m, m)
    for(i in (p+1):nrow(bteb))
    {
        Sigma1 <- Sigma1 + diagbt[,,i] + (diagbt[,,i]%*%(exp(DD) -
                                                         matrix(1, m, m))%*%diagbt[,,i])

    }
    Sigma2 <- crossprod(y%*%A)/(nrow(y)-p)
    Sigma <- Sigma2 + Sigma1/(nrow(y)-p)

    # Now find the correlations for this draw.
    tmp <- cov2cor(Sigma)
    C <- tmp[lower.tri(tmp, diag=FALSE)]

    return(list(bt, bteb, vech(Sigma), C, A))
}

# Load the posterior output, data, and compute the modes
library(foreign)

# Read in the data
d <- read.dta("dyadyr.dta")

# Set up the data time series we need for the DV
y <- ts(cbind(d$conflu, d$confls), start=c(1951, 1), freq=1)
colnames(y) <- c("US2SU", "SU2US")

# Set up the covariates we need for each equation
x <- ts(cbind(d$usmil, d$sumil, d$ppid), start=c(1951,1), freq=1)
colnames(x) <- c("US Military $", "SU Military $", "President")


##################################################################
# Now compute the effects across the sub-samples of the data; let's
# just use the first chain
##################################################################
# Load posterior sample
load("MVPLN-superpower-1.RData")

system.time(ms1 <- apply(x2[[1]], 1, bapvarmoments, x=x, y=y,
                         m=2, p=1))

rm(x2); gc()

##################################################################
# Now unwrap the output into a simple matrix so we can analyze the
# posterior of these quantities

ms1 <- matrix(unlist(ms1), ncol=length(unlist(ms1[[1]])), byrow=TRUE)

coef.names <- c("bt1", "bt2",
                paste("bteb", 1:56, sep=""),
                "Sigma[1,1]", "Sigma[2,1]", "Sigma[2,2]",
                "C",
                paste("A", 1:4, sep=""))

colnames(ms1) <- coef.names

# Now summarize the results

tmp1 <- summary(mcmc(ms1[,c(1:2,59:66)]), quantiles=c(0.05, 0.5,
                                          0.95))

# Uncomment to plot to device

#plot(mcmc(ms1[,c(1:2,59:66)]), ask=TRUE)

# Print out the results
print(tmp1)

# Now compute the AR coefficients so we can compare them over the
# periods.  The means and medians are very similar here.
m <- 2
p <- 1

AR1 <- array(tmp1$statistics[7:10, 1], c(m,m,p))

# Now dump these into a file.
library(xtable)

apply(AR1, 3, xtable)

# Now do the same for the Sigma matrices.  Do these for the median
# since some of these have bad outliers.
Sigma1 <- round(xpnd(matrix(tmp1$quantiles[3:5, 2])), 1)

xtable(Sigma1)

# Now do the same for the correlation matrices.  Do these for the median
# since some of these have bad outliers.

C1 <- tmp1$quantile[6, 2]


###################################################################
# Now compute the IRF and make sure it is scaled properly per the
# notes
###################################################################

library(MSBVAR)

"MVPLN.irf" <- function(A0, ar.coef, bteb, nsteps, m, p)
  {

      B<-array(0,c(m,m,max(nsteps,p)))     # Make an array for the AR coefs

      for(l in 1:(min(p,nsteps)))
      { B[,,l]<-ar.coef[,,l] }

      mhat<-matrix(0,ncol=m*m,nrow=nsteps)
      dim(mhat)<-c(m,m,nsteps)             # Make an array to hold IRF
      btebm <- matrix(bteb, m, m)
      mhat[,,1]<-A0*btebm                  # Identification condition
                                           # for IRF
#      mhat[,,1] <- A0

      for(i in 2:nsteps)                   # Compute the IRF -- note
                                        # the scale factor in here
      { for(j in 1:(i-1))
        { mhat[,,i]<- (mhat[,,i] + (mhat[,,(i-j)]%*%B[,,j])) }
    }
      impulses <- aperm(mhat, c(3,1,2))
      dim(impulses) <- c((m^2)*nsteps, 1)
      return(impulses)
}


plot.mc.irf.VAR <- function (x, method, component, probs, varnames = NULL, ...)
{
    mc.impulse <- x
    m <- sqrt(dim(mc.impulse)[3])
    nsteps <- dim(mc.impulse)[2]
    draws <- dim(mc.impulse)[1]
    irf.ci <- array(0, c(nsteps, length(probs) + 1, m^2))
    if (method == "Percentile") {
        eigen.sum <- 0
        for (i in 1:m^2) {
            irf.bands <- t(apply(mc.impulse[, , i], 2, quantile,
                probs))
            irf.mean <- apply(mc.impulse[, , i], 2, median)
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
        }
    }
    if (method == "Normal Approximation") {
        eigen.sum <- 0
        for (i in 1:m^2) {
            irf.mean <- apply(mc.impulse[, , i], 2, mean)
            irf.var <- apply(mc.impulse[, , i], 2, var)
            irf.bands <- irf.mean + matrix(rep(qnorm(probs),
                each = nsteps), nrow = nsteps) * irf.var
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
        }
    }
    if (method == "Sims-Zha1") {
        eigen.sum <- matrix(0, m^2, nsteps)
        for (i in 1:m^2) {
            decomp <- eigen(var(mc.impulse[, , i]), symmetric = T)
            W <- decomp$vectors
            lambda <- decomp$values
            irf.mean <- apply(mc.impulse[, , i], 2, mean)
            irf.bands <- irf.mean + W[, component] * matrix(rep(qnorm(probs),
                each = nsteps), nrow = nsteps) * sqrt(lambda[component])
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
            eigen.sum[i, ] <- 100 * lambda/sum(lambda)
        }
    }
    if (method == "Sims-Zha2") {
        eigen.sum <- matrix(0, m^2, nsteps)
        for (i in 1:m^2) {
            decomp <- eigen(var(mc.impulse[, , i]), symmetric = T)
            W <- decomp$vectors
            lambda <- decomp$values
            gammak <- mc.impulse[, , i] * (W[component, ])
            gammak.quantiles <- t(apply(gammak, 2, quantile,
                probs = probs))
            irf.mean <- apply(mc.impulse[, , i], 2, median)
            irf.bands <- irf.mean + gammak.quantiles
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
            eigen.sum[i, ] <- 100 * lambda/sum(lambda)
        }
    }
    if (method == "Sims-Zha3") {
        eigen.sum <- matrix(0, m^2, m^2 * nsteps)
        stacked.irf <- array(mc.impulse, c(draws, m^2 * nsteps))
        decomp <- eigen(var(stacked.irf), symmetric = T)
        W <- decomp$vectors
        lambda <- decomp$values
        gammak <- stacked.irf * W[component, ]
        gammak.quantiles <- apply(gammak, 2, quantile, probs)
        irf.mean <- matrix(apply(stacked.irf, 2, median), nrow = length(probs),
            ncol = dim(stacked.irf)[2], byrow = T)
        irf.bands <- irf.mean + gammak.quantiles
        irf.ci <- array((rbind(irf.bands, irf.mean[1, ])), c(length(probs) +
            1, nsteps, m^2))
        irf.ci <- aperm(irf.ci, c(2, 1, 3))
        eigen.sum <- 100 * lambda/sum(lambda)
    }
    minmax <- matrix(0, nrow = m, ncol = 2)
    within.plots <- apply(irf.ci, 3, range)
    tmp <- (c(1:m^2)%%m)
    tmp[tmp == 0] <- m
    indices <- sort(tmp, index.return = T)$ix
    dim(indices) <- c(m, m)
    for (i in 1:m) {
        minmax[i, ] <- range(within.plots[, indices[, i]])
    }
    j <- 1
    par(mfcol = c(m, m), mai = c(0.25, 0.25, 0.15, 0.25), omi = c(0.15,
        0.75, 1, 0.15))
    for (i in 1:m^2) {
        lims <- ifelse((i - m)%%m == 0, m, (i - m)%%m)
        ts.plot(irf.ci[, , i], gpars = list(xlab = "", ylab = "",
            ylim = minmax[lims, ]))
        abline(h = 0)
        if (i <= m) {
            mtext(varnames[i], side = 2, line = 3)
        }
        if ((i - 1)%%m == 0) {
            mtext(varnames[j], side = 3, line = 2)
            j <- j + 1
        }
    }
    mtext("Response in", side = 2, line = 3, outer = T)
    mtext("Shock to", side = 3, line = 3, outer = T)
    if (method == "Sims-Zha1" | method == "Sims-Zha2") {
        if (is.null(varnames) == T)
            varnames <- paste("V", seq(1:m), sep = "")
        shock.name <- rep(varnames, m)
        response.name <- rep(varnames, each = m)
        eigen.sum <- cbind(shock.name, response.name, as.data.frame(eigen.sum))
        colnames(eigen.sum) <- c("Shock", "Response", paste("Component",
            seq(1:nsteps)))
    }
    if (method == "Sims-Zha3") {
        names(eigen.sum) <- c(paste("Component", seq(1:m^2 *
            nsteps)))
    }
    invisible(list(responses = irf.ci, eigenvector.fractions = eigen.sum))
}

####################################################################
# Set parameters
####################################################################
num.ar <- (m^2)*p
nsteps <- 6

####################################################################
# Do this for each sub-sample of data
####################################################################

irf1 <- matrix(NA, nrow(ms1), ncol=(m^2)*nsteps)

for(i in 1:nrow(ms1))
{
    A0 <- t(chol(xpnd(ms1[i,59:61])))
    ar.coef <- array(ms1[i, 63:66], c(m,m,p))
    bteb <- apply(matrix(ms1[i, 3:58], ncol=2), 2, mean)
    irf1[i,] <- MVPLN.irf(A0, ar.coef, bteb, nsteps, m, p)
}

# Reorder the responses for the plotting function
dim(irf1) <- c(nrow(irf1), nsteps, m^2)
class(irf1) <- "mc.irf.VAR"

####################################################################
# Do some cleanup before running the impulse responses -- we no longer
# need the coefs for the computation after doing the irfs
####################################################################
save(ms1, file="BAPVAR-VAR1-moments.RData")
rm(ms1)
gc(); gc();


####################################################################
# Now plot all of the output, start with the percentile methods
####################################################################
pdf(file="USSU-irfs-percentile-VAR1.pdf", width=8, height=8)
nm <- c("US to Soviet", "Soviet to US")

plot.mc.irf.VAR(irf1, probs=c(0.16, 0.84),
                method="Percentile", component=1, varnames=nm)
dev.off()

####################################################################
# Now do the same with SZ methods
####################################################################
pdf(file="USSU-irfs-SZ2-VAR1.pdf", width=6, height=6)

irf1.SZ2 <- plot.mc.irf.VAR(irf1, probs=c(0.16, 0.84),
                            method="Sims-Zha2", component=1, varnames=nm)
dev.off()

#########
pdf(file="USSU-irfs-SZ3-VAR1.pdf", width=6, height=6)

irf1.SZ3 <- plot.mc.irf.VAR(irf1, probs=c(0.16, 0.84),
                            method="Sims-Zha3", component=1, varnames=nm)

dev.off()

# Print out the eigenvector fractions for the decompositions of the
# IRF variances.
print(irf1.SZ2$eigenvector.fractions[,1:3])
print(irf1.SZ3$eigenvector.fractions)


save.image("USSU-dynamics-VAR1.RData")

q(save="no")

