# BAPVAR-superpower-multicore.R -- BaPVAR models with a dynamic process.
# This is the superpower rivalry example that replicates King (1989).
# This file does the multicore / processor version.
#
# Initial file for setting things up to run the
# model through rjags.
#
# Patrick T. Brandt
#
# 20110508 : Initial version
# 20110523 : Multicore version

# Load libraries
library(zoo)
library(rjags)
library(foreign)

# Read in the data
d <- read.dta("dyadyr.dta")

# Set up the data time series we need for the DV
y <- ts(cbind(d$conflu, d$confls), start=c(1951, 1), freq=1)
colnames(y) <- c("US2SU", "SU2US")

# Set up the covariates we need for each equation
x <- ts(cbind(d$usmil, d$sumil, d$ppid), start=c(1951,1), freq=1)
colnames(x) <- c("US Military $", "SU Military $", "President")

# Set up data and constants for JAGS calls from R
ytmp <- as.data.frame(cbind(y,x))
colnames(ytmp) <- c(colnames(y), "USMIL", "SUMIL", "PREZ")

rownames(ytmp) <- NULL
N <- dim(ytmp)[[1]]
m <- 2                # Number of DVs

# Output the data
attach(ytmp)
dump(c("US2SU", "SU2US", "USMIL", "SUMIL", "PREZ", "N", "m"),
     file="MVPLN-data-superpower.R")
detach(ytmp)

# Set up the prior initializations for each, since
# these need to be scaled relative to the data and the number of
# lags.  Here we will write out the reults into a file since we are
# going to read them back in for each core in the computation.
# Assign the initial conditions for the sampler -- these are
# in the neighborhood of the prior.  Note that this is going to
# work for any number of lags since the dump of the output is
# inside this final loop.  We will disperse the values computed above
# so that we can properly use posterior diagnostic checks like
# Gelman-Rubin diagnostics.  Also, note that we do this twice and put
# the results into a list.  This is so we can run multiple chains.

n.chains <- 2
TT <- nrow(y)

set.seed(8086)

# Need a list of inits for P, b, AR coefficients, exogenous
# coefficients.  Want one for each chain.

inits1 <- inits2 <- inits3 <- vector(mode="list", n.chains)

# BaPVAR(1)
inits1[[1]] <- list(P=solve(crossprod(matrix(rnorm(TT*m), TT, m))),
                    b=matrix(rnorm(m, mean=log(apply(ytmp[,1:2], 2,
                                      mean)), sd=4), 1, m),
                    A1=diag(runif(m, 0.5, 1)),
                    xog=matrix(rnorm(m*3), m, 3), .RNG.seed=609,
                    .RNG.name="base::Wichmann-Hill")

inits1[[2]] <- list(P=solve(crossprod(matrix(rnorm(TT*m), TT, m))),
                    b=matrix(rnorm(m, mean=log(apply(ytmp[,1:2], 2,
                                      mean)), sd=4), 1, m),
                    A1=diag(runif(m, 0.5, 1)),
                    xog=matrix(rnorm(m*3), m, 3), .RNG.seed=602,
                    .RNG.name="base::Wichmann-Hill")

# BaPVAR(2)
inits2[[1]] <- list(P=solve(crossprod(matrix(rnorm(TT*m), TT, m))),
                    b=matrix(rnorm(m, mean=log(apply(ytmp[,1:2], 2,
                                      mean)), sd=4), 1, m),
                    A1=diag(runif(m, 0.5, 1)), A2=matrix(0, m, m),
                    xog=matrix(rnorm(m*3), m, 3), .RNG.seed=815,
                    .RNG.name="base::Wichmann-Hill")

inits2[[2]] <- list(P=solve(crossprod(matrix(rnorm(TT*m), TT, m))),
                    b=matrix(rnorm(m, mean=log(apply(ytmp[,1:2], 2,
                                      mean)), sd=4), 1, m),
                    A1=diag(runif(m, 0.5, 1)), A2=matrix(0, m, m),
                    xog=matrix(rnorm(m*3), m, 3), .RNG.seed=1110,
                    .RNG.name="base::Wichmann-Hill")

# BaPVAR(3)
inits3[[1]] <- list(P=solve(crossprod(matrix(rnorm(TT*m), TT, m))),
                    b=matrix(rnorm(m, mean=log(apply(ytmp[,1:2], 2,
                                      mean)), sd=4), 1, m),
                    A1=diag(runif(m, 0.5, 1)), A2=matrix(0, m, m),
                    A3=matrix(0, m, m),
                    xog=matrix(rnorm(m*3), m, 3), .RNG.seed=105,
                    .RNG.name="base::Wichmann-Hill")

inits3[[2]] <- list(P=solve(crossprod(matrix(rnorm(TT*m), TT, m))),
                    b=matrix(rnorm(m, mean=log(apply(ytmp[,1:2], 2,
                                      mean)), sd=4), 1, m),
                    A1=diag(runif(m, 0.5, 1)), A2=matrix(0, m, m),
                    A3=matrix(0, m, m),
                    xog=matrix(rnorm(m*3), m, 3), .RNG.seed=9865,
                    .RNG.name="base::Wichmann-Hill")

# Now write out the initializations into a list
inits <- list(inits1, inits2, inits3)


# Define p.max = max number of lags.
p.max <- 3

# Define the list over which we will operate using snow
plist <- p.max:1

# Baseline call to a model / sampler -- we dispatch this to multiple
# cores.
MVPLNmodel <- function(i, plist, bugprefix, data, inits,
                       chains=2, n.adapt=1000, n.final=1000)
{
    pidx <- plist[i]

    # Use the prefixes and the input list to get the input names.
    bug <- paste(bugprefix, "-", pidx, ".bug", sep="")

    # Burn in
    x1 <- jags.model(file=bug, data=read.data(data),
                     inits=inits[[i]],
                     n.chains=chains, n.adapt)

    # Final draws and monitoring
    monitorwhat <- c("b", "xog", "Sigma", "C", paste("A", 1:pidx, sep=""))

    x2 <- coda.samples(x1,  monitorwhat, n.final)

    # Write out the results and clean up the memory
    save(x2, file=paste("MVPLN-superpower-", pidx, ".RData", sep=""))
    rm(x2); gc(); gc();

    # Compute the DIC and penalized deviance measures
    dic <- dic.samples(x1, n.iter=n.final, type="pD")

    # Return the DIC for later manipuation and computations so we know
    # which posterior sample is optimal

    return(list(dic=dic, p=pidx))
}

# Running one process as a check
## tmp <- MVPLNmodel(1, plist,
##                   bugprefix="MVPLN-VAR",
##                   data="MVPLN-data-superpower.R",
##                   initsprefix="MVPLN-inits",
##                   chains=2, n.adapt=100, n.final=100)

######################################################################
# Now set up the loops for the multiprocessor computation of the
# models.  Will have each slave do a two-chain run.  So the BaP-VAR(p)
# models will run on p-slaves.
######################################################################
#library(Rmpi)
library(snow)

# Set up the cluster and pass it the packages it needs and initialize
# the random number seeds

nodes <- 3
cl <- makeCluster(spec=nodes, type="SOCK")
clusterEvalQ(cl, library(rjags))
clusterSetupRNGstream(cl)

# Set up calling all of this over multiple nodes.  Each node works
# with one of the nodes datasets.

out <- clusterApplyLB(cl, 1:length(plist),
                      MVPLNmodel,
                      plist=plist,
                      bugprefix="MVPLN-VAR",
                      data="MVPLN-data-superpower.R",
                      inits=inits,
                      chains=2, n.adapt=100000, n.final=250000)

# Shut down the cluster
stopCluster(cl)

# Check the models for fit
print(out)

# Save results and quit nicely -- will summarize in a table later

save(out, file="MVPLN-VAR-DIC.RData")

