# BAPVAR-superpower-multipliers.R
#
# Posterior dynamic multipliers for the covariates for the
# multi-chain, multicore model.
#
# Patrick T. Brandt
#
# 20110609 : Initial version -- borrows from the IRF analysis code,
#            but does the analysis over the exogenous coefficients
#            only.

library(coda)

# Load the posterior samples
load("MVPLN-superpower-1.RData")

# Get the summaries for the coefficients for the exogenous variables
plot(mcmc.list(x2[[1]][,15:20], x2[[2]][,15:20]))

# Get the impact or initial multipliers -- these are for the xog[]
# coefficients from the model
impacts <- mcmc.list(exp(x2[[1]][,15:20]), exp(x2[[2]][,15:20]))
plot(impacts)

summary(impacts)

# Now compute the dynamic multipliers.  We will do this same way as
# the IRFs with a function that does it for one draw and then we will
# apply this function to the entire posterior sample and summarize.


# Here's the function that will compute the relevant multipler(s) for
# a single draw of the BaP-VAR
# ar.coef   : m x m x p array of the AR(p) coefficients
# exog.coef : m x 1 matrix of the exogenous coefficients to be
#             analyzed
# m         : m -- number of equations
# p         : number of lags
# nsteps    : number of steps over which to compute the responses.

BaPVAR.multiplier <- function(ar.coef, exog.coef, m, p, nsteps)
{
    B<-array(0,c(m,m,max(nsteps,p)))     # Make an array for the AR coefs

    for(l in 1:(min(p,nsteps))) { B[,,l]<-ar.coef[,,l] }

    # Object to store output over a shock
    mhat<-matrix(0,ncol=m*m,nrow=nsteps)
    dim(mhat)<-c(m,m,nsteps)             # Make an array to hold IRF


    # We are going to want to do this step over each of the effects.
    mhat[,,1] <- diag(exp(exog.coef))

    for(i in 2:nsteps)                   # Compute the IRF -- note
                                         # the scale factor in here
    { for(j in 1:(i-1))
      { mhat[,,i]<- (mhat[,,i] + (mhat[,,(i-j)]%*%B[,,j])) }
  }
    impulses <- aperm(mhat, c(3,1,2))

    # Combine to get the total effects of the unit shock
    return(apply(impulses, 2, rowSums))
}

# Set up constants
m <- 2
p <- 1
nsteps <- 6

# Now set up a function to compute the parallel version across the
# exogenous regressors.  This lets us do 3 times the work in the same
# amount of time.

BaPVARmult.parallel <- function(j, x2, m, p, nsteps)
{  # Compute some constants
    ncoef <- (m^2)*p
    ncoef2 <- ncoef + 2*m^2 + 1
    ncoef3 <- length(x2[[1]][1,])

    # Storage
    mults <- matrix(0, nrow(x2[[1]]), nsteps*m)

    # Main loop for each core to execute
    for(i in 1:nrow(x2[[1]]))
    {
        A <- array(x2[[1]][i,1:ncoef], c(m, m, p))

        # Note how we use j here to index the coefficients
        exog.coef <- matrix(x2[[1]][i,(ncoef2+m):ncoef3], nrow=m)

        # Compute the multipliers
        tmp <- BaPVAR.multiplier(A, exog.coef[,j], m, p, nsteps)
        mults[i,] <- tmp

        # Return from each node
     }
    return(mults)
}


idx <- 1:3

# Now set this up to run in parallel
library(snow)

# Set up the nodes and pass over the function to compute the multipliers
nodes <- max(idx)
cl <- makeCluster(spec=nodes, type="SOCK")
clusterExport(cl, "BaPVAR.multiplier")

# Now call the nodes to compute the multipliers
system.time(mult.out <- clusterApply(cl, idx, fun=BaPVARmult.parallel,
                                     x2=x2, m=m, p=p, nsteps=nsteps))

# End the cluster
stopCluster(cl)


######################################################################
# Now summarize the results with some plots
######################################################################

# Plot the dynamic multipler for the effect of a on unit change in the
# covariate for each equation.
pdf(file="superpowerexog.pdf", width=6, height=4)
par(mfcol=c(2, 3), mar=c(2, 4, 1, 1), cex.lab=0.8)
plot(ts(t(apply(mult.out[[1]][,1:nsteps], 2,
                quantile, probs=c(0.05, 0.5, 0.95)))),
     plot.type="single",
     ylab="US2SU MIDS response to 1 unit change in USMIL",
     lty=c(2,1,2), ylim=c(0,5))

plot(ts(t(apply(mult.out[[1]][,(nsteps+1):(m*nsteps)], 2,
                quantile, probs=c(0.05, 0.5, 0.95)))),
     plot.type="single",
                ylab="SU2US MIDS response to 1 unit change in USMIL",
     lty=c(2,1,2), ylim=c(0,5))

plot(ts(t(apply(mult.out[[2]][,1:nsteps], 2,
                quantile, probs=c(0.05, 0.5, 0.95)))),
     plot.type="single",
                ylab="US2SU MIDS response to 1 unit change in SUMIL",
     lty=c(2,1,2), ylim=c(0,5))

plot(ts(t(apply(mult.out[[2]][,(nsteps+1):(m*nsteps)], 2,
                quantile, probs=c(0.05, 0.5, 0.95)))),
     plot.type="single",
                ylab="SU2US MIDS response to 1 unit change in SUMIL",
     lty=c(2,1,2), ylim=c(0,5))

plot(ts(t(apply(mult.out[[3]][,1:nsteps], 2,
                quantile, probs=c(0.05, 0.5, 0.95)))),
     plot.type="single",
                ylab="US2SU MIDS response to change from D to R",
     lty=c(2,1,2), ylim=c(0,5))

plot(ts(t(apply(mult.out[[3]][,(nsteps+1):(m*nsteps)], 2,
                quantile, probs=c(0.05, 0.5, 0.95)))),
     plot.type="single",
                ylab="SU2US MIDS response to change from D to R",
     lty=c(2,1,2), ylim=c(0,5))

dev.off()

save.image(file="BAPVAR-superpower-multipliers.RData")

q(save="no")


########################################################################
#
# Test the code for a single multipler -- this is the code that was
# the basis for the BaPVAR.multiplier code
#
########################################################################


## # Temp object for testing code
## post <- x2[[1]][1,]

## m <- 2
## p <- 1
## ncoef <- (m^2)*p
## ncoef2 <- ncoef + 2*m^2 + 1
## ncoef3 <- length(post)
## nsteps <- 6

## ar.coef <- A <- array(post[1:ncoef], c(m, m, p))

## # Version with covariates other than an intercept
## exog.coef <- matrix(post[ncoef2:ncoef3], nrow=m)

## # Compute the responses
## B<-array(0,c(m,m,max(nsteps,p)))     # Make an array for the AR coefs

## for(l in 1:(min(p,nsteps))) { B[,,l]<-ar.coef[,,l] }

## # Object to store output over a shock
## mhat<-matrix(0,ncol=m*m,nrow=nsteps)
## dim(mhat)<-c(m,m,nsteps)             # Make an array to hold IRF


## # We are going to want to do this step over each of the effects.
## mhat[,,1] <- diag(exp(exog.coef[,2]))

## for(i in 2:nsteps)                   # Compute the IRF -- note
##                                         # the scale factor in here
## { for(j in 1:(i-1))
##   { mhat[,,i]<- (mhat[,,i] + (mhat[,,(i-j)]%*%B[,,j])) }
## }
## impulses <- aperm(mhat, c(3,1,2))
## dim(impulses) <- c((m^2)*nsteps, 1)


