# MVPLN-superpower.R -- MVPLN model with a VAR dynamic process.
# This is the superpower rivalry example that replicates King (1989).
#
# THIS IS THE TOY EXAMPLE THAT SHOWS HOW THINGS ARE SET UP AND CAN BE
# RUN SERIALLY.  NOT FOR REAL PRODUCTION USE, ONLY AS AN EXAMPLE.
#
# Initial file for setting things up to run the
# model through rjags.
#
# Patrick T. Brandt
#
# 20110508 : Initial version
#

library(zoo)
library(rjags)
library(foreign)

# Read in the data
d <- read.dta("dyadyr.dta")

# Set up the data time series we need for the DV
y <- ts(cbind(d$conflu, d$confls), start=c(1951, 1), freq=1)
colnames(y) <- c("US2SU", "SU2US")

# Set up the covariates we need for each equation
x <- ts(cbind(d$usmil, d$sumil, d$ppid), start=c(1951,1), freq=1)
colnames(x) <- c("US Military $", "SU Military $", "President")

# Plot DVs and their distributions
pdf(file="superpowerdatadiag.pdf", width=6, height=4)
par(mfrow=c(2,3))
plot(y[,1], ylab="US -> SU")
acf(y[,1], main="")
hist(y[,1], main="", xlab="")

plot(y[,2], ylab="SU -> US")
acf(y[,2], main="")
hist(y[,2], main="", xlab="")
dev.off()

# Set up data and constants for JAGS calls from R
ytmp <- as.data.frame(cbind(y,x))
colnames(ytmp) <- c(colnames(y), "USMIL", "SUMIL", "PREZ")

rownames(ytmp) <- NULL
N <- dim(ytmp)[[1]]
m <- 2                # Number of DVs

# Output the data
attach(ytmp)
dump(c("US2SU", "SU2US", "USMIL", "SUMIL", "PREZ", "N", "m"),
     file="MVPLN-data-superpower.R")
detach(ytmp)

# Set up the prior initializations.
# These need to be scaled relative to the data and the number of lags.
# Assign the initial conditions for the sampler -- these are
# in the neighborhood of the prior.  Note that this is going to
# work for any number of lags since the dump of the output is
# inside this final loop.  We will disperse the values computed above
# so that we can properly use posterior diagnostic checks like
# Gelman-Rubin diagnostics.  Also, note that we do this twice and put
# the results into a list.  This is so we can run multiple chains.

n.chains <- 2
TT <- nrow(y)

set.seed(8086)

# Need a list of inits for P, b, AR coefficients, exogenous
# coefficients.  Want one for each chain.

inits1 <- inits2 <- inits3 <- vector(mode="list", n.chains)

# BaPVAR(1)
inits1[[1]] <- list(P=solve(0.8*var(y)),
                    b=matrix(rnorm(m, mean=log(apply(ytmp[,1:2], 2,
                                      mean)), sd=4), 1, m),
                    A1=diag(m),
                    xog=matrix(0, m, 3), .RNG.seed=609,
                    .RNG.name="base::Wichmann-Hill")

inits1[[2]] <- list(P=solve(0.8*var(y)),
                    b=matrix(rnorm(m, mean=log(apply(ytmp[,1:2], 2,
                                      mean)), sd=4), 1, m),
                    A1=diag(m),
                    xog=matrix(0, m, 3), .RNG.seed=602,
                    .RNG.name="base::Wichmann-Hill")

# BaPVAR(2)
inits2[[1]] <- list(P=P=solve(0.8*var(y)),
                    b=matrix(rnorm(m, mean=log(apply(ytmp[,1:2], 2,
                                      mean)), sd=4), 1, m),
                    A1=diag(runif(m, 0.5, 1)), A2=matrix(0, m, m),
                    xog=matrix(rnorm(m*3), m, 3), .RNG.seed=815,
                    .RNG.name="base::Wichmann-Hill")

inits2[[2]] <- list(P=solve(0.8*var(y)),
                    b=matrix(rnorm(m, mean=log(apply(ytmp[,1:2], 2,
                                      mean)), sd=4), 1, m),
                    A1=diag(runif(m, 0.5, 1)), A2=matrix(0, m, m),
                    xog=matrix(rnorm(m*3), m, 3), .RNG.seed=1110,
                    .RNG.name="base::Wichmann-Hill")

# BaPVAR(3)
inits3[[1]] <- list(P=solve(0.8*var(y)),
                    b=matrix(rnorm(m, mean=log(apply(ytmp[,1:2], 2,
                                      mean)), sd=4), 1, m),
                    A1=diag(runif(m, 0.5, 1)), A2=matrix(0, m, m),
                    A3=matrix(0, m, m),
                    xog=matrix(rnorm(m*3), m, 3), .RNG.seed=105,
                    .RNG.name="base::Wichmann-Hill")

inits3[[2]] <- list(P=solve(0.8*var(y)),
                    b=matrix(rnorm(m, mean=log(apply(ytmp[,1:2], 2,
                                      mean)), sd=4), 1, m),
                    A1=diag(runif(m, 0.5, 1)), A2=matrix(0, m, m),
                    A3=matrix(0, m, m),
                    xog=matrix(rnorm(m*3), m, 3), .RNG.seed=9865,
                    .RNG.name="base::Wichmann-Hill")



# Now fit the baseline models for lags 1-3 and compute their DICs
# Set initial parameters for the MCMC runs
N1 <- 10000
N2 <- 10000

# Fit the models.  Could do this with snow or some parallel
# processing, but this is fast enough for this small dataset.

# BaPVAR(1)
basemodel1 <- jags.model(file="MVPLN-VAR-1.bug",
                         data=read.data("MVPLN-data-superpower.R"),
                         inits=inits1, n.chains=n.chains,
                         n.adapt=N1)

# Monitor / draw posterior after burnin
MVPLN.post1 <- coda.samples(basemodel1, c("b", "Sigma", "C", "A1", "xog"), N2)

# Compute the DIC
dic1 <- dic.samples(basemodel1, n.iter=N2, type="pD")

# BaPVAR(2)
basemodel2 <- jags.model(file="MVPLN-VAR-2.bug",
                        data=read.data("MVPLN-data-superpower.R"),
                        inits=inits2, n.chains=n.chains,
                        n.adapt=N1)

MVPLN.post2 <- coda.samples(basemodel2, c("b", "Sigma", "C", "A1", "A2"), N2)

dic2 <- dic.samples(basemodel2, n.iter=N2, type="pD")

# BaPVAR(3)
basemodel3 <- jags.model(file="MVPLN-VAR-3.bug",
                         data=read.data("MVPLN-data-superpower.R"),
                         inits=inits3, n.chains=n.chains,
                         n.adapt=N1)

MVPLN.post3 <- coda.samples(basemodel3,
                            c("b", "Sigma", "C", "A1", "A2", "A3"),
                            N2)

dic3 <- dic.samples(basemodel3, n.iter=N2, type="pD")

save.image("MVPLN-superpower.RData")


# Print out DICs for model fit statistics -- selects the BaP-VAR(1)!
print(dic1)
print(dic2)
print(dic3)

# Check convergence -- only do this for the non-redundant parameters
print(gelman.diag(MVPLN.post1[,c(1:4,9:10,12:14)]))
print(gelman.diag(MVPLN.post2[,c(1:8,13:14,16:18)]))
print(gelman.diag(MVPLN.post3[,c(1:12,17:18,20:22)]))

# Quit nicely
#q(save="no")
