# VAR-superpower.R
#
# Patrick T. Brandt
#
# Fits a VAR model to the superpower data example
#
#

# Load libraries
library(MSBVAR)
library(foreign)

# Read in the data
d <- read.dta("dyadyr.dta")

# Set up the data time series we need for the DV
y <- ts(cbind(log(d$conflu), log(d$confls)), start=c(1951, 1), freq=1)
colnames(y) <- c("US2SU", "SU2US")

# Set up the covariates we need for each equation
x <- ts(cbind(d$usmil, d$sumil, d$ppid), start=c(1951,1), freq=1)
colnames(x) <- c("US Military $", "SU Military $", "President")

# Lag specification check
print(var.lag.specification(y, lagmax=4))

# Fit the VAR
VAR1 <- reduced.form.var(y, p=1, z=x)

# Print out the VAR regression coefficients and credible intervals
coefs <- VAR1$Bhat
rownames(coefs) <- c("US2SU", "SU2US", "Intercept", "US Military $",
                     "SU Military $", "US President")
se <- matrix(sqrt(diag(kronecker(VAR1$mean.S, solve(VAR1$hstar)))),
             ncol=2)

lower1 <- coefs[,1] + qnorm(0.025)*se[,1]
upper1 <- coefs[,1] + qnorm(0.975)*se[,1]

lower2 <- coefs[,2] + qnorm(0.025)*se[,2]
upper2 <- coefs[,2] + qnorm(0.975)*se[,2]

tmp <- cbind(cbind(coefs[,1], lower1, upper1),
             cbind(coefs[,2], lower2, upper2))

library(xtable)
print(xtable(tmp, digits=3), file="VARcoefs.tex")

# Generate the impulse responses to compare to those from the
# BaP-VAR(1)
set.seed(123)
VAR1.irf <- mc.irf(VAR1, nstep=6, draws=250000)

# Plot the results
pdf(file="MVN-VAR-irfs-percentile.pdf", width=6, height=6)
plot(VAR1.irf, method=c("Percentile"), probs=c(0.16, 0.84))
dev.off()


######################################################################
#
# Here's the function that will compute the relevant multipler(s) for
# exogenous variables for a single draw of the parameters of a VAR
#
# ar.coef   : m x m x p array of the AR(p) coefficients
# exog.coef : m x 1 matrix of the exogenous coefficients to be
#             analyzed
# m         : m -- number of equations
# p         : number of lags
# nsteps    : number of steps over which to compute the responses.
#

VAR.multiplier <- function(ar.coef, exog.coef, m, p, nsteps)
{
    B<-array(0,c(m,m,max(nsteps,p)))     # Make an array for the AR coefs

    for(l in 1:(min(p,nsteps))) { B[,,l]<-ar.coef[,,l] }

    # Object to store output over a shock
    mhat<-matrix(0,ncol=m*m,nrow=nsteps)
    dim(mhat)<-c(m,m,nsteps)             # Make an array to hold IRF


    # We are going to want to do this step over each of the effects.
    mhat[,,1] <- diag(exog.coef)

    for(i in 2:nsteps)                   # Compute the IRF -- note
                                         # the scale factor in here
    { for(j in 1:(i-1))
      { mhat[,,i]<- (mhat[,,i] + (mhat[,,(i-j)]%*%B[,,j])) }
  }
    impulses <- aperm(mhat, c(3,1,2))

    # Combine to get the total effects of the unit shock
    return(apply(impulses, 2, rowSums))
}


# Now, generate the covariate multipler effects on the assumption of
# Gaussian errors and a linear VAR process.  This is a simple
# posterior simulation of the fitted VAR(1) model above.


N <- 250000
m <- 2
TT <- nrow(y) - m
k <- 6
p <- 1
mults <- array(0, c(N, k, 3*m))

# Other constant we need for the VAR multiplers loop below
XXinv <- solve(VAR1$hstar)
Sigma <- VAR1$mean.S
Sigmainv <- solve(Sigma)
B1 <- as.vector(VAR1$Bhat)


for (i in 1:N)
{
    # Simulate the VAR coefficients

    # Get the error covariance and draw
    Sigmadraw <- TT*solve(rwishart(1, TT, Sigmainv))
    SXX <- kronecker(Sigmadraw, XXinv)

    # Draw coefficients
    Bdraw <- matrix(B1 + t(chol(SXX))%*%rnorm(length(B1)), ncol=m)

    # Generate the responees

    ar.coef <- array(Bdraw[1:(m*p),], c(m,m,p))
    exog.coef <- Bdraw[((m*p)+2):nrow(Bdraw),]

    # Responses
    M1 <- VAR.multiplier(ar.coef, exog.coef[1,], m, p, k) # US Mil
    M2 <- VAR.multiplier(ar.coef, exog.coef[2,], m, p, k) # SU Mil
    M3 <- VAR.multiplier(ar.coef, exog.coef[3,], m, p, k) # POTUS

    # Save the repsonses

    mults[i,,1] <- M1[,1]
    mults[i,,2] <- M1[,2]
    mults[i,,3] <- M2[,1]
    mults[i,,4] <- M2[,2]
    mults[i,,5] <- M3[,1]
    mults[i,,6] <- M3[,2]

}


# Summarize the output so we can plot the multipliers
tmp <- apply(mults, c(2,3), mean)

# Get the quantiles and plot them.  These are the 90% quantiles like
# in the BaP-VAR example
multq <- apply(mults, c(2,3),g quantile, probs=c(0.05, 0.5, 0.95))

pdf(file="superpower-VAR-exog.pdf", width=6, height=4)
par(mfcol=c(2, 3), mar=c(2, 4, 1, 1), cex.lab=0.8)
plot(ts(t(multq[,,1])),
     plot.type="single",
     ylab="US2SU MIDS response to 1 unit change in USMIL",
     lty=c(2,1,2))
abline(h=0)

plot(ts(t(multq[,,2])),
     plot.type="single",
                ylab="SU2US MIDS response to 1 unit change in USMIL",
     lty=c(2,1,2))
abline(h=0)

plot(ts(t(multq[,,3])),
     plot.type="single",
                ylab="US2SU MIDS response to 1 unit change in SUMIL",
     lty=c(2,1,2))
abline(h=0)

plot(ts(t(multq[,,4])),
     plot.type="single",
                ylab="SU2US MIDS response to 1 unit change in SUMIL",
     lty=c(2,1,2))
abline(h=0)

plot(ts(t(multq[,,5])),
     plot.type="single",
                ylab="US2SU MIDS response to change from D to R",
     lty=c(2,1,2))
abline(h=0)

plot(ts(t(multq[,,6])),
     plot.type="single",
                ylab="SU2US MIDS response to change from D to R",
     lty=c(2,1,2))
abline(h=0)
dev.off()

# Quit gracefully
q(save="no")
