# MVPLN-Gaussian-VAR.R -- Gaussian VAR analysis.
#
# 20111111 : Initial version
# 20111114 : Added the sampling of the error covariances

# Load libraries
library(zoo)
library(MSBVAR)

# Read in the data
load("MonthlySeries1968-2008.RData")

# Set up the data series
y <- cbind(x.m[[1]][,10], x.m[[2]][,10],
           x.m[[3]][,10], x.m[[4]][,10])

colnames(y) <- c("O", "M", "B", "PP")

# Split DV series into periods defined by the breakpoints
y1 <- window(as.ts(y), end=c(1973,1))
y2 <- window(as.ts(y), start=c(1973,2), end=c(1979,12))
y3 <- window(as.ts(y), start=c(1980,1), end=c(1989,11))
y4 <- window(as.ts(y), start=c(1989, 12), end=c(2001,9))
y5 <- window(as.ts(y), start=c(2001, 10))

# Check on lag length specifications
print(var.lag.specification(y1, lagmax=4))

print(var.lag.specification(y2, lagmax=4))

print(var.lag.specification(y3, lagmax=4))

print(var.lag.specification(y4, lagmax=6))

print(var.lag.specification(y5, lagmax=6))

# Now fit the VAR models for each data series
m1 <- reduced.form.var(y1, p=1)
m2 <- reduced.form.var(y2, p=1)
m3 <- reduced.form.var(y3, p=1)
m4 <- reduced.form.var(y4, p=4)
m5 <- reduced.form.var(y5, p=1)

# Generate impulse responses
set.seed(9982)

# Constants for the simulations
draws <- 100000
nsteps <- 6

irf1 <- mc.irf(m1, nsteps, draws)
irf2 <- mc.irf(m2, nsteps, draws)
irf3 <- mc.irf(m3, nsteps, draws)
irf4 <- mc.irf(m4, nsteps, draws)
irf5 <- mc.irf(m5, nsteps, draws)

# Plot the results
nm <- c("Officials", "Military", "Business", "Private Parties")

pdf(file="Gaussian-irfs-percentile-monthly.pdf", width=8, height=8)

plot(irf1, probs=c(0.16, 0.84), method="Percentile", varnames=nm)
plot(irf2, probs=c(0.16, 0.84), method="Percentile", varnames=nm)
plot(irf3, probs=c(0.16, 0.84), method="Percentile", varnames=nm)
plot(irf4, probs=c(0.16, 0.84), method="Percentile", varnames=nm)
plot(irf5, probs=c(0.16, 0.84), method="Percentile", varnames=nm)

dev.off()

# Get the covariances / correlations of the errors for each model
# Quick function to replace lower triangular portion with correlations

covcor <- function(x)
{
    tmp <- cov2cor(x)
    m <- dim(x)[1]
    xout <- x
    for (i in 1:m)
    {
        for(j in 1:m)
        {
            if(i==j) xout[i,j] <- x[i,j]
            if(j<i) xout[i,j] <- x[i,j]
            if(i>j) xout[i,j] <- tmp[i,j]
        }
    }
    return(xout)
}

# Now sample the posterior error covariance densities.
#
# 1) Get proper df.
# 2) Sample from a Wishart
# 3) Convert to covariance / correlations we need for table.
#
#
set.seed(9981)
# Sample 1
df1 <- nrow(y1) - m1$pfit$ncoef
tmp <- rwishart(draws, df1, m1$mean.S/df1)
c1 <- t(apply(tmp, 3, covcor))
rm(tmp)

# Sample 2
df2 <- nrow(y2) - m2$pfit$ncoef
tmp <- rwishart(draws, df2, m2$mean.S/df2)
c2 <- t(apply(tmp, 3, covcor))
rm(tmp)

# Sample 3
df3 <- nrow(y3) - m3$pfit$ncoef
tmp <- rwishart(draws, df3, m3$mean.S/df3)
c3 <- t(apply(tmp, 3, covcor))
rm(tmp)

# Sample 4
df4 <- nrow(y4) - m4$pfit$ncoef
tmp <- rwishart(draws, df4, m4$mean.S/df4)
c4 <- t(apply(tmp, 3, covcor))
rm(tmp)

# Sample 5
df5 <- nrow(y5) - m5$pfit$ncoef
tmp <- rwishart(draws, df5, m5$mean.S/df5)
c5 <- t(apply(tmp, 3, covcor))
rm(tmp)


# Now summarize the error covariances
c1.sum <- summary(mcmc(c1), quantiles=c(0.05, 0.5, 0.95))
c2.sum <- summary(mcmc(c2), quantiles=c(0.05, 0.5, 0.95))
c3.sum <- summary(mcmc(c3), quantiles=c(0.05, 0.5, 0.95))
c4.sum <- summary(mcmc(c4), quantiles=c(0.05, 0.5, 0.95))
c5.sum <- summary(mcmc(c5), quantiles=c(0.05, 0.5, 0.95))

print(round(c1.sum$quantiles[,c(1,3)], 2))
print(round(c2.sum$quantiles[,c(1,3)], 2))
print(round(c3.sum$quantiles[,c(1,3)], 2))
print(round(c4.sum$quantiles[,c(1,3)], 2))
print(round(c5.sum$quantiles[,c(1,3)], 2))

library(xtable)
xtable(matrix(c1.sum$quantiles[,2], 4, 4), digits=2)
xtable(matrix(c2.sum$quantiles[,2], 4, 4), digits=2)
xtable(matrix(c3.sum$quantiles[,2], 4, 4), digits=2)
xtable(matrix(c4.sum$quantiles[,2], 4, 4), digits=2)
xtable(matrix(c5.sum$quantiles[,2], 4, 4), digits=2)


# Generate a plot of the histograms over the data series and variables
pdf(file="histograms.pdf", width=8, height=8)
par(mfrow=c(5,4), mar=c(2,4,2,1), omi=c(0,1,0.25,0))
for(i in 1:4)
{ hist(y1[,i], main=nm[i], xlab="", xlim=c(0,70), ylim=c(0,120),
       ylab=ifelse(i==1, "Frequency", ""))
  if(i==1) mtext("1968:1-1973:1", side=2, line=5, cex=0.7)
}

for(i in 1:4)
{ hist(y2[,i], main="", xlab="", xlim=c(0,70), ylim=c(0,120),
       ylab=ifelse(i==1, "Frequency", ""))
  if(i==1) mtext("1973:2-1979:12", side=2, line=5, cex=0.7)
}

for(i in 1:4)
{ hist(y3[,i], main="", xlab="", xlim=c(0,70), ylim=c(0,120),
       ylab=ifelse(i==1, "Frequency", ""))
  if(i==1) mtext("1980:1-1989:11", side=2, line=5, cex=0.7)
}
mtext("Subsample", side=2, line=6, cex=0.8)

for(i in 1:4)
{ hist(y4[,i], main="", xlab="", xlim=c(0,70), ylim=c(0,120),
       ylab=ifelse(i==1, "Frequency", ""))
  if(i==1) mtext("1989:12-2001:9", side=2, line=5, cex=0.7)
}

for(i in 1:4)
{ hist(y5[,i], main="", xlab="", xlim=c(0,70), ylim=c(0,120),
       ylab=ifelse(i==1, "Frequency", ""))
  if(i==1) mtext("2001:10-2008:12", side=2, line=5, cex=0.7)
}

dev.off()

# Save the results

q(save="no")
