# MVPLN-VAR-full-dynamics.R
#
# Computes the dynamic multipliers for MVPLN-VAR model for the full
# data sample.
#
#
# Patrick T. Brandt
#
# 20100820 : Initial version
#

# Now compute the effects across the data; let's
# just use the first chain.  First load the data object x2 that has
# the two chains for the full sample, one lag model.

load("MVPLN-full-1.RData")

# Compute the impulse responses based on each of the posterior
# draws. Initial function computes one IRF.  This will then be looped
# over to do the full IRF.

######################################################################
# Function for the period specific IRF and the full sample IRFs.
# x      = one draw of the model parameters from the posterior output
# nsteps = number of periods over which to compute the IRF
# m      = number of equations
# p      = number of lags used in the BaP-VAR(p) model
# pd     = index for the dummy variable period so we can properly scale
#          the response for that period.
######################################################################

"MVPLN.irf" <- function(x, nsteps, m, p, pd=0)
{
    ncoef <- (m^2)*p
    ncoef2 <- ncoef + 2*m^2 + 1
    ncoef3 <- ncoef2 + m - 1

    A <- matrix(x[1:ncoef], m*p, m, byrow=TRUE)
    b <- (matrix(x[ncoef2:ncoef3], m, 1))
    d <- t(matrix(x[(ncoef3+1):length(x)], m,m, byrow=TRUE))

    DD <- matrix(x[(ncoef + m^2 + 1):(ncoef2-1)], m, m)

    bt <- exp(0.5*diag(DD))

    # Now check which time dummies we need.
    if(pd==0) {
        bteb <- as.vector(bt*exp(b)) } else {
            bteb <-as.vector(bt*exp(b+d[,pd])) }

    diagbt <- diag(bteb)

    # Compute VCV of error process
    Sigma <- (diagbt%*%(exp(DD) - matrix(1, m, m))%*%diagbt)

    # Cholesky decomposition
    A0 <- t(chol(Sigma))

    # Make an array for the AR coefs for the IRF
    B <- array(0,c(m,m,max(nsteps,p)))

    # This is specific to an AR(1) -- need the AR coefs in an array
    # for more than this case
    for(l in 1:(min(p,nsteps))) { B[,,l] <- A }

    # Make an array to hold IRF
    mhat <- array(0, c(m,m,nsteps))

    # Set up the correct constant and deterministic components
    btebm <- matrix(bteb, m, m)

    # Initial Identification condition for IRF of BaP-VAR(1)
    mhat[,,1]<-A0*btebm

    for(i in 2:nsteps)                   # Compute the IRF -- note
                                        # the scale factor in here
    { for(j in 1:(i-1))
      { mhat[,,i]<- (mhat[,,i] + (mhat[,,(i-j)]%*%B[,,j])) }
  }
    impulses <- aperm(mhat, c(3,1,2))
    dim(impulses) <- c((m^2)*nsteps, 1)
    return(impulses)
}


####################################################################
# Set parameters
####################################################################
nsteps <- 6
p <- 1
m <- 4

####################################################################
# Do this for each sub-sample of data
####################################################################

irf0 <- matrix(NA, nrow(x2[[1]]), ncol=(m^2)*nsteps)
irf4 <- irf3 <- irf2 <- irf1 <- irf0

for(i in 1:nrow(x2[[1]]))
{
    irf0[i,] <- MVPLN.irf(x2[[1]][i,], nsteps, m, p, pd=0)
    irf1[i,] <- MVPLN.irf(x2[[1]][i,], nsteps, m, p, pd=1)
    irf2[i,] <- MVPLN.irf(x2[[1]][i,], nsteps, m, p, pd=2)
    irf3[i,] <- MVPLN.irf(x2[[1]][i,], nsteps, m, p, pd=3)
    irf4[i,] <- MVPLN.irf(x2[[1]][i,], nsteps, m, p, pd=4)
}

# Clean up to get more memory back
rm(x2); gc(); gc()

####################################################################
# Reorder and class the responses for the plotting function
####################################################################
dim(irf0) <- c(100000, nsteps, m^2)
class(irf0) <- "mc.irf.VAR"

dim(irf1) <- c(100000, nsteps, m^2)
class(irf1) <- "mc.irf.VAR"

dim(irf2) <- c(100000, nsteps, m^2)
class(irf2) <- "mc.irf.VAR"

dim(irf3) <- c(100000, nsteps, m^2)
class(irf3) <- "mc.irf.VAR"

dim(irf4) <- c(100000, nsteps, m^2)
class(irf4) <- "mc.irf.VAR"


###################################################################
# Now plot the IRF and make sure it is scaled properly per the
# notes.  This plotting function is lifted from the MSBVAR R pkg.
###################################################################

plot.mc.irf.VAR <- function (x, method, component, probs, varnames = NULL, ...)
{
    mc.impulse <- x
    m <- sqrt(dim(mc.impulse)[3])
    nsteps <- dim(mc.impulse)[2]
    draws <- dim(mc.impulse)[1]
    irf.ci <- array(0, c(nsteps, length(probs) + 1, m^2))
    if (method == "Percentile") {
        eigen.sum <- 0
        for (i in 1:m^2) {
            irf.bands <- t(apply(mc.impulse[, , i], 2, quantile,
                probs))
            irf.mean <- apply(mc.impulse[, , i], 2, median)
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
        }
    }
    if (method == "Normal Approximation") {
        eigen.sum <- 0
        for (i in 1:m^2) {
            irf.mean <- apply(mc.impulse[, , i], 2, mean)
            irf.var <- apply(mc.impulse[, , i], 2, var)
            irf.bands <- irf.mean + matrix(rep(qnorm(probs),
                each = nsteps), nrow = nsteps) * irf.var
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
        }
    }
    if (method == "Sims-Zha1") {
        eigen.sum <- matrix(0, m^2, nsteps)
        for (i in 1:m^2) {
            decomp <- eigen(var(mc.impulse[, , i]), symmetric = T)
            W <- decomp$vectors
            lambda <- decomp$values
            irf.mean <- apply(mc.impulse[, , i], 2, mean)
            irf.bands <- irf.mean + W[, component] * matrix(rep(qnorm(probs),
                each = nsteps), nrow = nsteps) * sqrt(lambda[component])
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
            eigen.sum[i, ] <- 100 * lambda/sum(lambda)
        }
    }
    if (method == "Sims-Zha2") {
        eigen.sum <- matrix(0, m^2, nsteps)
        for (i in 1:m^2) {
            decomp <- eigen(var(mc.impulse[, , i]), symmetric = T)
            W <- decomp$vectors
            lambda <- decomp$values
            gammak <- mc.impulse[, , i] * (W[component, ])
            gammak.quantiles <- t(apply(gammak, 2, quantile,
                probs = probs))
            irf.mean <- apply(mc.impulse[, , i], 2, median)
            irf.bands <- irf.mean + gammak.quantiles
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
            eigen.sum[i, ] <- 100 * lambda/sum(lambda)
        }
    }
    if (method == "Sims-Zha3") {
        eigen.sum <- matrix(0, m^2, m^2 * nsteps)
        stacked.irf <- array(mc.impulse, c(draws, m^2 * nsteps))
        decomp <- eigen(var(stacked.irf), symmetric = T)
        W <- decomp$vectors
        lambda <- decomp$values
        gammak <- stacked.irf * W[component, ]
        gammak.quantiles <- apply(gammak, 2, quantile, probs)
        irf.mean <- matrix(apply(stacked.irf, 2, median), nrow = length(probs),
            ncol = dim(stacked.irf)[2], byrow = T)
        irf.bands <- irf.mean + gammak.quantiles
        irf.ci <- array((rbind(irf.bands, irf.mean[1, ])), c(length(probs) +
            1, nsteps, m^2))
        irf.ci <- aperm(irf.ci, c(2, 1, 3))
        eigen.sum <- 100 * lambda/sum(lambda)
    }
    minmax <- matrix(0, nrow = m, ncol = 2)
    within.plots <- apply(irf.ci, 3, range)
    tmp <- (c(1:m^2)%%m)
    tmp[tmp == 0] <- m
    indices <- sort(tmp, index.return = T)$ix
    dim(indices) <- c(m, m)
    for (i in 1:m) {
        minmax[i, ] <- range(within.plots[, indices[, i]])
    }
    j <- 1
    par(mfcol = c(m, m), mai = c(0.25, 0.25, 0.15, 0.25), omi = c(0.15,
        0.75, 1, 0.15))
    for (i in 1:m^2) {
        lims <- ifelse((i - m)%%m == 0, m, (i - m)%%m)
        ts.plot(irf.ci[, , i], gpars = list(xlab = "", ylab = "",
            ylim = minmax[lims, ]))
        abline(h = 0)
        if (i <= m) {
            mtext(varnames[i], side = 2, line = 3)
        }
        if ((i - 1)%%m == 0) {
            mtext(varnames[j], side = 3, line = 2)
            j <- j + 1
        }
    }
    mtext("Response in", side = 2, line = 3, outer = T)
    mtext("Shock to", side = 3, line = 3, outer = T)
    if (method == "Sims-Zha1" | method == "Sims-Zha2") {
        if (is.null(varnames) == T)
            varnames <- paste("V", seq(1:m), sep = "")
        shock.name <- rep(varnames, m)
        response.name <- rep(varnames, each = m)
        eigen.sum <- cbind(shock.name, response.name, as.data.frame(eigen.sum))
        colnames(eigen.sum) <- c("Shock", "Response", paste("Component",
            seq(1:nsteps)))
    }
    if (method == "Sims-Zha3") {
        names(eigen.sum) <- c(paste("Component", seq(1:m^2 *
            nsteps)))
    }
    invisible(list(responses = irf.ci, eigenvector.fractions = eigen.sum))
}



####################################################################
# Now plot all of the output, start with the percentile methods.
# These are all going to have the same dynamics, just subject to the
# shifts for the time periods
####################################################################
nm <- c("Officials", "Military", "Business", "Private Parties")

pdf(file="irfs-percentile0.pdf", width=6, height=6)
plot.mc.irf.VAR(irf0, probs=c(0.16, 0.84),
                method="Percentile", component=1, varnames=nm)
dev.off()

pdf(file="irfs-percentile1.pdf", width=6, height=6)
plot.mc.irf.VAR(irf1, probs=c(0.16, 0.84),
                method="Percentile", component=1, varnames=nm)
dev.off()

pdf(file="irfs-percentile2.pdf", width=6, height=6)
plot.mc.irf.VAR(irf2, probs=c(0.16, 0.84),
                method="Percentile", component=1, varnames=nm)
dev.off()

pdf(file="irfs-percentile3.pdf", width=6, height=6)
plot.mc.irf.VAR(irf3, probs=c(0.16, 0.84),
                method="Percentile", component=1, varnames=nm)
dev.off()

pdf(file="irfs-percentile4.pdf", width=6, height=6)
plot.mc.irf.VAR(irf4, probs=c(0.16, 0.84),
                method="Percentile", component=1, varnames=nm)
dev.off()

####################################################################
# Now do the same with SZ methods
####################################################################
pdf(file="irfs-Sims-Zha2-0.pdf", width=6, height=6)
plot.mc.irf.VAR(irf0, probs=c(0.16, 0.84),
                method="Sims-Zha2", component=1, varnames=nm)
dev.off()

pdf(file="irfs-Sims-Zha2-1.pdf", width=6, height=6)
plot.mc.irf.VAR(irf1, probs=c(0.16, 0.84),
                method="Sims-Zha2", component=1, varnames=nm)
dev.off()

pdf(file="irfs-Sims-Zha2-2.pdf", width=6, height=6)
plot.mc.irf.VAR(irf2, probs=c(0.16, 0.84),
                method="Sims-Zha2", component=1, varnames=nm)
dev.off()

pdf(file="irfs-Sims-Zha2-3.pdf", width=6, height=6)
plot.mc.irf.VAR(irf3, probs=c(0.16, 0.84),
                method="Sims-Zha2", component=1, varnames=nm)
dev.off()

pdf(file="irfs-Sims-Zha2-4.pdf", width=6, height=6)
plot.mc.irf.VAR(irf4, probs=c(0.16, 0.84),
                method="Sims-Zha2", component=1, varnames=nm)
dev.off()




#########
pdf(file="irfs-SZ3-monthly.pdf", width=6, height=6)

irf1.SZ3 <- plot.mc.irf.VAR(irf1, probs=c(0.16, 0.84),
                            method="Sims-Zha3", component=1, varnames=nm)
dev.off()

# Print out the eigenvector fractions for the decompositions of the
# IRF variances.
print(irf1.SZ2$eigenvector.fractions[,1:3])

print(irf1.SZ3$eigenvector.fractions)

save.image("MVPLN-VAR-monthly-dynamics.RData")

q(save="no")


