# MVPLN-VAR-full.R -- MVPLN model with a VAR dynamic process.
# Initial file for setting things up to run the
# model through rjags.

# 20100722 : Initial version for the full sample model

library(zoo)
library(rjags)

# Read in the data
load("MonthlySeries1968-2008.RData")

# Set up the count data series
y <- cbind(x.m[[1]][,10], x.m[[2]][,10],
           x.m[[3]][,10], x.m[[4]][,10])

colnames(y) <- c("O", "M", "B", "PP")

# Create dummy varaibles for the time periods / breaks
x1 <- ifelse(index(y) > "Jan 1973" & index(y) < "Jan 1980", 1, 0)
x2 <- ifelse(index(y) > "Dec 1979" & index(y) < "Dec 1989", 1, 0)
x3 <- ifelse(index(y) > "Nov 1989" & index(y) < "Oct 2001", 1, 0)
x4 <- ifelse(index(y) > "Sep 2001", 1, 0)

# Now set up the data for the looped multi-processor runs.  Much of
# this is prep for JAGS

# Define p.max = max number of lags.  This will then be used as part of the
# definition of the number of parameters, priors and initializations.
# Later this will be part of defining the inputs to the models and the
# output.

p.max <- 3

# Set up data and constants for JAGS calls from R
ytmp <- as.data.frame(cbind(y, x1, x2, x3, x4))
rownames(ytmp) <- NULL
N <- dim(ytmp)[[1]]
m <- dim(y)[[2]]

# Output the data
attach(ytmp)
dump(c("O", "M", "B", "PP", "x1", "x2", "x3", "x4", "N", "m"),
     file="MVPLN-data-full.R")
detach(ytmp)

# Set up the prior initializations for each sub-sample, since
# these need to be scaled relative to the data and the number of
# lags

# Prior precision
P <- solve(cov(y))

# Prior intercepts
b <- matrix(log(apply(y, 2, mean)), 1, m)

# Prior intercept shifts
s <- matrix(0, 4, 4)

# Generate the number of lag matrices after the first one.
ARnames <- paste("A", 1:p.max, sep="")

# Now assign the initial conditions for the sampler -- these are
# in the neighborhood of the prior.  Note that this is going to
# work for any number of lags since the dump of the output is
# inside this final loop

for(j in 1:p.max)
{
    if(j==1){ assign(ARnames[j], 0.9*diag(m)) } else {

        assign(ARnames[j], matrix(0, m, m))
    }

    # Output the initial conditions for each of the samplers for each
    # sampler.  Files are named by sample index-lag index

    dump(c("P", "b", "s", ARnames[1:j]),
         file=paste("MVPLN-inits-full-", j, ".R", sep=""))
}



# Define the list over which we will operate using snow / Rmpi
plist <- p.max:1

# Baseline call to a model / sampler -- we dispatch this to multiple
# cores.
MVPLNmodel <- function(i, plist, bugprefix, data, initsprefix,
                       chains=2, n.adapt=1000, n.final=1000)
{
    pidx <- plist[i]

    # Use the prefixes and the input list to get the input names.
    bug <- paste(bugprefix, "-", pidx, ".bug", sep="")
    inits <- paste(initsprefix, "-", pidx, ".R", sep="")

    # Burn in
    x1 <- jags.model(file=bug, data=read.data(data),
                     inits=read.data(inits),
                     n.chains=chains, n.adapt)

    # Final draws and monitoring
    monitorwhat <- c("b", "s", "Sigma", "C", paste("A", 1:pidx, sep=""))

    x2 <- coda.samples(x1,  monitorwhat, n.final)

    # Write out the results and clean up the memory
    save(x2, file=paste("MVPLN-full-", pidx, ".RData", sep=""))
    rm(x2); gc(); gc();

    # Compute the DIC and penalized deviance measures
    dic <- dic.samples(x1, n.iter=n.final, type="pD")

    # Return the DIC for later manipuation and computations so we know
    # which posterior sample is optimal

    return(list(dic=dic, p=pidx))
}

## # Running one process as a check
## tmp <- MVPLNmodel(3, plist,
##                   bugprefix="MVPLN-VAR-full",
##                   data="MVPLN-data-full.R",
##                   initsprefix="MVPLN-inits-full",
##                   chains=2, n.adapt=100, n.final=100)


# Now set up the loops for the multiprocessor computation of the
# models.

library(Rmpi)
library(snow)

# Set up the cluster and pass it the packages it needs and initialize
# the random number seeds

set.seed(9806)
nodes <- 3
cl <- makeCluster(spec=nodes, type="SOCK")
clusterEvalQ(cl, library(rjags))
clusterSetupRNGstream(cl)

# Set up calling all of this over multiple nodes.  Each node works
# with one of the nodes datasets.

out <- clusterApplyLB(cl, 1:length(plist),
                      MVPLNmodel,
                      plist=plist,
                      bugprefix="MVPLN-VAR-full",
                      data="MVPLN-data-full.R",
                      initsprefix="MVPLN-inits-full",
                      chains=2, n.adapt=30000, n.final=100000)

# Shut down the cluster
stopCluster(cl)

# Save results and quit nicely -- will summarize in a table later

save(out, file="MVPLN-VAR-full-DIC.RData")

mpi.quit(save="no")
