# MVPLN-VAR-monthly-dynamics.R
#
# Computes the dynamic multipliers for MVPLN-VAR models.
#
#
# Patrick T. Brandt
#
# 20100319 : Initial version
# 20100426 : Updated to use new variance computations.
# 20100712 : Updated code to work for VAR(1) models selected by DIC
#            criteria from among 15 specifications.
# 20101104 : Revised for shorter IRF computations
# 20101214 : Added the computation of the correlations for the RF of
#            the BaP-VAR in the musigma() function, renamed now to
#            bapvarmoments().
# 20110103 : Updated covariances for the BaP-VAR moments computation

library(coda)

# vech function for efficiently sorting symmetric matrices (like Sigma)

vech <- function (x)
{
    x <- as.matrix(x)
    if (dim(x)[1] != dim(x)[2]) {
        stop("Non-square matrix passed to vech().\n")
    }
    output <- x[lower.tri(x, diag = TRUE)]
    dim(output) <- NULL
    return(output)
}

# xpnd function for reconstructing symmetric matrices from vech

xpnd <- function (x, nrow = NULL)
{
    dim(x) <- NULL
    if (is.null(nrow))
        nrow <- (-1 + sqrt(1 + 8 * length(x)))/2
    output <- matrix(0, nrow, nrow)
    output[lower.tri(output, diag = TRUE)] <- x
    hold <- output
    hold[upper.tri(hold, diag = TRUE)] <- 0
    output <- output + t(hold)
    return(output)
}

# bapvarmoments() -- computes the mean and variance for the
# measurement equation of the model.  As an aside, it also computes
# the correlation matrix of the errors.
bapvarmoments <- function(x, y, m, p)
{
    ncoef <- (m^2)*p
    ncoef2 <- ncoef + 2*m^2 + 1
    ncoef3 <- length(x)

    A <- matrix(x[1:ncoef], m*p, m, byrow=TRUE)
    eb <- exp(matrix(x[ncoef2:ncoef3], m, 1))
    DD <- matrix(x[(ncoef + m^2 + 1):(ncoef2-1)], m, m)

    bt <- exp(0.5*diag(DD))
    bteb <- as.vector(bt*eb)
    diagbt <- diag(bteb)

    # Use vech here so we can store relevant parts of symmetric pd
    # matrices
    Sigma1 <- diagbt + (diagbt%*%(exp(DD) - matrix(1, m, m))%*%diagbt)
    Sigma2 <- crossprod(y%*%A)/nrow(y)
    Sigma <- Sigma2 + Sigma1

    # Now find the correlations for this draw.
    tmp <- cov2cor(Sigma)
    C <- tmp[lower.tri(tmp, diag=FALSE)]
    return(list(bt, bteb, vech(Sigma), C, A))
}

# Load the posterior output and compute the modes

# Read in the data
load("MonthlySeries1968-2008.RData")

# Set up the data series
y <- cbind(x.m[[1]][,10], x.m[[2]][,10],
           x.m[[3]][,10], x.m[[4]][,10])

colnames(y) <- c("O", "M", "B", "PP")

# Split DV series into periods defined by the breakpoints
y1 <- window(as.ts(y), end=c(1973,1))
y2 <- window(as.ts(y), start=c(1973,2), end=c(1979,12))
y3 <- window(as.ts(y), start=c(1980,1), end=c(1989,11))
y4 <- window(as.ts(y), start=c(1989, 12), end=c(2001,9))
y5 <- window(as.ts(y), start=c(2001, 10))

# Now compute the effects across the sub-samples of the data; let's
# just use the first chain

load("MVPLN-monthly-1-1.RData")
ms1 <- apply(x2[[1]], 1, bapvarmoments, y=y1, m=4, p=1)
rm(x2); gc()

load("MVPLN-monthly-2-1.RData")
ms2 <- apply(x2[[1]], 1, bapvarmoments, y=y2, m=4, p=1)
rm(x2); gc()

load("MVPLN-monthly-3-1.RData")
ms3 <- apply(x2[[1]], 1, bapvarmoments, y=y3, m=4, p=1)
rm(x2); gc()

load("MVPLN-monthly-4-1.RData")
ms4 <- apply(x2[[1]], 1, bapvarmoments, y=y4, m=4, p=1)
rm(x2); gc()

load("MVPLN-monthly-5-1.RData")
ms5 <- apply(x2[[1]], 1, bapvarmoments, y=y5, m=4, p=1)
rm(x2); gc()

# Now unwrap the output into a simple matrix so we can analyze the
# posterior of these quantities

ms1 <- matrix(unlist(ms1), ncol=length(unlist(ms1[[1]])), byrow=TRUE)
ms2 <- matrix(unlist(ms2), ncol=length(unlist(ms2[[1]])), byrow=TRUE)
ms3 <- matrix(unlist(ms3), ncol=length(unlist(ms3[[1]])), byrow=TRUE)
ms4 <- matrix(unlist(ms4), ncol=length(unlist(ms4[[1]])), byrow=TRUE)
ms5 <- matrix(unlist(ms5), ncol=length(unlist(ms5[[1]])), byrow=TRUE)


coef.names <- c("bt1", "bt2", "bt3", "bt4",
                "bteb1", "bteb2", "bteb3", "bteb4",
                "Sigma[1,1]", "Sigma[2,1]",
                "Sigma[3,1]", "Sigma[4,1]",
                "Sigma[2,2]", "Sigma[3,2]",
                "Sigma[4,2]", "Sigma[3,3]",
                "Sigma[4,3]", "Sigma[4,4]",
                "C[2,1]", "C[3,1]", "C[4,1]",
                "C[3,2]", "C[4,2]", "C[4,3]",
                paste("A", 1:16, sep=""))

colnames(ms1) <- coef.names
colnames(ms2) <- coef.names
colnames(ms3) <- colnames(ms4) <- coef.names
colnames(ms5) <- coef.names

# Now summarize the results

tmp1 <- summary(mcmc(ms1), quantiles=c(0.05, 0.5, 0.95))
tmp2 <- summary(mcmc(ms2), quantiles=c(0.05, 0.5, 0.95))
tmp3 <- summary(mcmc(ms3), quantiles=c(0.05, 0.5, 0.95))
tmp4 <- summary(mcmc(ms4), quantiles=c(0.05, 0.5, 0.95))
tmp5 <- summary(mcmc(ms5), quantiles=c(0.05, 0.5, 0.95))

# Print out the results
print(tmp1)
print(tmp2)
print(tmp3)
print(tmp4)
print(tmp5)

# Now compute the AR coefficients so we can compare them over the
# periods.  The means and medians are very similar here.
m <- 4
p <- 1

AR1 <- array(tmp1$statistics[25:40, 1], c(m,m,p))
AR2 <- array(tmp2$statistics[25:40, 1], c(m,m,p))
AR3 <- array(tmp3$statistics[25:40, 1], c(m,m,p))
AR4 <- array(tmp4$statistics[25:40, 1], c(m,m,p))
AR5 <- array(tmp5$statistics[25:40, 1], c(m,m,p))

# Now dump these into a file.
library(xtable)

apply(AR1, 3, xtable)
apply(AR2, 3, xtable)
apply(AR3, 3, xtable)
apply(AR4, 3, xtable)
apply(AR5, 3, xtable)

# Now do the same for the Sigma matrices.  Do these for the median
# since some of these have bad outliers.
Sigma1 <- round(xpnd(matrix(tmp1$quantile[9:18, 2])), 1)
Sigma2 <- round(xpnd(matrix(tmp2$quantile[9:18, 2])), 1)
Sigma3 <- round(xpnd(matrix(tmp3$quantile[9:18, 2])), 1)
Sigma4 <- round(xpnd(matrix(tmp4$quantile[9:18, 2])), 1)
Sigma5 <- round(xpnd(matrix(tmp5$quantile[9:18, 2])), 1)

xtable(Sigma1)
xtable(Sigma2)
xtable(Sigma3)
xtable(Sigma4)
xtable(Sigma5)

# Now do the same for the correlation matrices.  Do these for the median
# since some of these have bad outliers.

C1 <- round(xpnd(matrix(tmp1$quantile[19:24, 2])), 1)
C2 <- round(xpnd(matrix(tmp2$quantile[19:24, 2])), 1)
C3 <- round(xpnd(matrix(tmp3$quantile[19:24, 2])), 1)
C4 <- round(xpnd(matrix(tmp4$quantile[19:24, 2])), 1)
C5 <- round(xpnd(matrix(tmp5$quantile[19:24, 2])), 1)

xtable(C1)
xtable(C2)
xtable(C3)
xtable(C4)
xtable(C5)

###################################################################
# Now compute the IRF and make sure it is scaled properly per the
# notes
###################################################################

library(MSBVAR)

"MVPLN.irf" <- function(A0, ar.coef, bteb, nsteps, m, p)
  {

      B<-array(0,c(m,m,max(nsteps,p)))     # Make an array for the AR coefs

      for(l in 1:(min(p,nsteps)))
      { B[,,l]<-ar.coef[,,l] }

      mhat<-matrix(0,ncol=m*m,nrow=nsteps)
      dim(mhat)<-c(m,m,nsteps)             # Make an array to hold IRF
      btebm <- matrix(bteb, m, m)
      mhat[,,1]<-A0*btebm                  # Identification condition
                                           # for IRF
#      mhat[,,1] <- A0

      for(i in 2:nsteps)                   # Compute the IRF -- note
                                        # the scale factor in here
      { for(j in 1:(i-1))
        { mhat[,,i]<- (mhat[,,i] + (mhat[,,(i-j)]%*%B[,,j])) }
    }
      impulses <- aperm(mhat, c(3,1,2))
      dim(impulses) <- c((m^2)*nsteps, 1)
      return(impulses)
}


plot.mc.irf.VAR <- function (x, method, component, probs, varnames = NULL, ...)
{
    mc.impulse <- x
    m <- sqrt(dim(mc.impulse)[3])
    nsteps <- dim(mc.impulse)[2]
    draws <- dim(mc.impulse)[1]
    irf.ci <- array(0, c(nsteps, length(probs) + 1, m^2))
    if (method == "Percentile") {
        eigen.sum <- 0
        for (i in 1:m^2) {
            irf.bands <- t(apply(mc.impulse[, , i], 2, quantile,
                probs))
            irf.mean <- apply(mc.impulse[, , i], 2, median)
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
        }
    }
    if (method == "Normal Approximation") {
        eigen.sum <- 0
        for (i in 1:m^2) {
            irf.mean <- apply(mc.impulse[, , i], 2, mean)
            irf.var <- apply(mc.impulse[, , i], 2, var)
            irf.bands <- irf.mean + matrix(rep(qnorm(probs),
                each = nsteps), nrow = nsteps) * irf.var
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
        }
    }
    if (method == "Sims-Zha1") {
        eigen.sum <- matrix(0, m^2, nsteps)
        for (i in 1:m^2) {
            decomp <- eigen(var(mc.impulse[, , i]), symmetric = T)
            W <- decomp$vectors
            lambda <- decomp$values
            irf.mean <- apply(mc.impulse[, , i], 2, mean)
            irf.bands <- irf.mean + W[, component] * matrix(rep(qnorm(probs),
                each = nsteps), nrow = nsteps) * sqrt(lambda[component])
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
            eigen.sum[i, ] <- 100 * lambda/sum(lambda)
        }
    }
    if (method == "Sims-Zha2") {
        eigen.sum <- matrix(0, m^2, nsteps)
        for (i in 1:m^2) {
            decomp <- eigen(var(mc.impulse[, , i]), symmetric = T)
            W <- decomp$vectors
            lambda <- decomp$values
            gammak <- mc.impulse[, , i] * (W[component, ])
            gammak.quantiles <- t(apply(gammak, 2, quantile,
                probs = probs))
            irf.mean <- apply(mc.impulse[, , i], 2, median)
            irf.bands <- irf.mean + gammak.quantiles
            irf.ci[, , i] <- cbind(irf.bands, irf.mean)
            eigen.sum[i, ] <- 100 * lambda/sum(lambda)
        }
    }
    if (method == "Sims-Zha3") {
        eigen.sum <- matrix(0, m^2, m^2 * nsteps)
        stacked.irf <- array(mc.impulse, c(draws, m^2 * nsteps))
        decomp <- eigen(var(stacked.irf), symmetric = T)
        W <- decomp$vectors
        lambda <- decomp$values
        gammak <- stacked.irf * W[component, ]
        gammak.quantiles <- apply(gammak, 2, quantile, probs)
        irf.mean <- matrix(apply(stacked.irf, 2, median), nrow = length(probs),
            ncol = dim(stacked.irf)[2], byrow = T)
        irf.bands <- irf.mean + gammak.quantiles
        irf.ci <- array((rbind(irf.bands, irf.mean[1, ])), c(length(probs) +
            1, nsteps, m^2))
        irf.ci <- aperm(irf.ci, c(2, 1, 3))
        eigen.sum <- 100 * lambda/sum(lambda)
    }
    minmax <- matrix(0, nrow = m, ncol = 2)
    within.plots <- apply(irf.ci, 3, range)
    tmp <- (c(1:m^2)%%m)
    tmp[tmp == 0] <- m
    indices <- sort(tmp, index.return = T)$ix
    dim(indices) <- c(m, m)
    for (i in 1:m) {
        minmax[i, ] <- range(within.plots[, indices[, i]])
    }
    j <- 1
    par(mfcol = c(m, m), mai = c(0.25, 0.25, 0.15, 0.25), omi = c(0.15,
        0.75, 1, 0.15))
    for (i in 1:m^2) {
        lims <- ifelse((i - m)%%m == 0, m, (i - m)%%m)
        ts.plot(irf.ci[, , i], gpars = list(xlab = "", ylab = "",
            ylim = minmax[lims, ]))
        abline(h = 0)
        if (i <= m) {
            mtext(varnames[i], side = 2, line = 3)
        }
        if ((i - 1)%%m == 0) {
            mtext(varnames[j], side = 3, line = 2)
            j <- j + 1
        }
    }
    mtext("Response in", side = 2, line = 3, outer = T)
    mtext("Shock to", side = 3, line = 3, outer = T)
    if (method == "Sims-Zha1" | method == "Sims-Zha2") {
        if (is.null(varnames) == T)
            varnames <- paste("V", seq(1:m), sep = "")
        shock.name <- rep(varnames, m)
        response.name <- rep(varnames, each = m)
        eigen.sum <- cbind(shock.name, response.name, as.data.frame(eigen.sum))
        colnames(eigen.sum) <- c("Shock", "Response", paste("Component",
            seq(1:nsteps)))
    }
    if (method == "Sims-Zha3") {
        names(eigen.sum) <- c(paste("Component", seq(1:m^2 *
            nsteps)))
    }
    invisible(list(responses = irf.ci, eigenvector.fractions = eigen.sum))
}

####################################################################
# Set parameters
####################################################################
num.ar <- (m^2)*p
nsteps <- 6

####################################################################
# Do this for each sub-sample of data
####################################################################

irf1 <- matrix(NA, nrow(ms1), ncol=(m^2)*nsteps)

for(i in 1:nrow(ms1))
{
    A0 <- t(chol(xpnd(ms1[i,9:18])))
    ar.coef <- array(ms1[i, 25:40], c(m,m,p))
    bteb <- ms1[i, 5:8]
    irf1[i,] <- MVPLN.irf(A0, ar.coef, bteb, nsteps, m, p)
}

# Reorder the responses for the plotting function
dim(irf1) <- c(100000, nsteps, m^2)
class(irf1) <- "mc.irf.VAR"

irf2 <- matrix(NA, nrow(ms2), ncol=(m^2)*nsteps)

for(i in 1:nrow(ms2))
{
    A0 <- t(chol(xpnd(ms2[i,9:18])))
    ar.coef <- array(ms2[i, 25:40], c(m,m,p))
    bteb <- ms2[i, 5:8]
    irf2[i,] <- MVPLN.irf(A0, ar.coef, bteb, nsteps, m, p)
}

# Reorder the responses for the plotting function
dim(irf2) <- c(100000, nsteps, m^2)
class(irf2) <- "mc.irf.VAR"

irf3 <- matrix(NA, nrow(ms3), ncol=(m^2)*nsteps)

for(i in 1:nrow(ms3))
{
    A0 <- t(chol(xpnd(ms3[i,9:18])))
    ar.coef <- array(ms3[i, 25:40], c(m,m,p))
    bteb <- ms3[i, 5:8]
    irf3[i,] <- MVPLN.irf(A0, ar.coef, bteb, nsteps, m, p)
}

# Reorder the responses for the plotting function
dim(irf3) <- c(100000, nsteps, m^2)
class(irf3) <- "mc.irf.VAR"

irf4 <- matrix(NA, nrow(ms4), ncol=(m^2)*nsteps)

for(i in 1:nrow(ms4))
{
    A0 <- t(chol(xpnd(ms4[i,9:18])))
    ar.coef <- array(ms4[i, 25:40], c(m,m,p))
    bteb <- ms4[i, 5:8]
    irf4[i,] <- MVPLN.irf(A0, ar.coef, bteb, nsteps, m, p)
}

# Reorder the responses for the plotting function
dim(irf4) <- c(100000, nsteps, m^2)
class(irf4) <- "mc.irf.VAR"


irf5 <- matrix(NA, nrow(ms5), ncol=(m^2)*nsteps)

for(i in 1:nrow(ms5))
{
    A0 <- t(chol(xpnd(ms5[i,9:18])))
    ar.coef <- array(ms5[i, 25:40], c(m,m,p))
    bteb <- ms5[i, 5:8]
    irf5[i,] <- MVPLN.irf(A0, ar.coef, bteb, nsteps, m, p)
}

# Reorder the responses for the plotting function
dim(irf5) <- c(100000, nsteps, m^2)
class(irf5) <- "mc.irf.VAR"

####################################################################
# Do some cleanup before running the impulse responses -- we no longer
# need the coefs for the computation after doing the irfs
####################################################################
save(ms1, ms2, ms3, ms4, ms5, file="MVPLN-VAR-monthly-qi-VAR1.RData")
rm(ms1, ms2, ms3, ms4, ms5)
gc(); gc();


####################################################################
# Now plot all of the output, start with the percentile methods
####################################################################
pdf(file="irfs-percentile-monthly-VAR1.pdf", width=8, height=8)
nm <- c("Officials", "Military", "Business", "Private Parties")

plot.mc.irf.VAR(irf1, probs=c(0.16, 0.84),
                method="Percentile", component=1, varnames=nm)

plot.mc.irf.VAR(irf2, probs=c(0.16, 0.84),
                method="Percentile", component=1, varnames=nm)

plot.mc.irf.VAR(irf3, probs=c(0.16, 0.84),
                method="Percentile", component=1, varnames=nm)

plot.mc.irf.VAR(irf4, probs=c(0.16, 0.84),
                method="Percentile", component=1, varnames=nm)

plot.mc.irf.VAR(irf5, probs=c(0.16, 0.84),
                method="Percentile", component=1, varnames=nm)
dev.off()

####################################################################
# Now do the same with SZ methods
####################################################################
pdf(file="irfs-SZ2-monthly-VAR1.pdf", width=6, height=6)

irf1.SZ2 <- plot.mc.irf.VAR(irf1, probs=c(0.16, 0.84),
                            method="Sims-Zha2", component=1, varnames=nm)

irf2.SZ2 <- plot.mc.irf.VAR(irf2, probs=c(0.16, 0.84),
                            method="Sims-Zha2", component=1, varnames=nm)

irf3.SZ2 <- plot.mc.irf.VAR(irf3, probs=c(0.16, 0.84),
                            method="Sims-Zha2", component=1, varnames=nm)

irf4.SZ2 <- plot.mc.irf.VAR(irf4, probs=c(0.16, 0.84),
                            method="Sims-Zha2", component=1, varnames=nm)

irf5.SZ2 <- plot.mc.irf.VAR(irf5, probs=c(0.16, 0.84),
                            method="Sims-Zha2", component=1, varnames=nm)
dev.off()

#########
pdf(file="irfs-SZ3-monthly-VAR1.pdf", width=6, height=6)

irf1.SZ3 <- plot.mc.irf.VAR(irf1, probs=c(0.16, 0.84),
                            method="Sims-Zha3", component=1, varnames=nm)

irf2.SZ3 <- plot.mc.irf.VAR(irf2, probs=c(0.16, 0.84),
                            method="Sims-Zha3", component=1, varnames=nm)

irf3.SZ3 <- plot.mc.irf.VAR(irf3, probs=c(0.16, 0.84),
                            method="Sims-Zha3", component=1, varnames=nm)

irf4.SZ3 <- plot.mc.irf.VAR(irf4, probs=c(0.16, 0.84),
                            method="Sims-Zha3", component=1, varnames=nm)

irf5.SZ3 <- plot.mc.irf.VAR(irf5, probs=c(0.16, 0.84),
                            method="Sims-Zha3", component=1, varnames=nm)
dev.off()

# Print out the eigenvector fractions for the decompositions of the
# IRF variances.
print(irf1.SZ2$eigenvector.fractions[,1:3])
print(irf2.SZ2$eigenvector.fractions[,1:3])
print(irf3.SZ2$eigenvector.fractions[,1:3])
print(irf4.SZ2$eigenvector.fractions[,1:3])
print(irf5.SZ2$eigenvector.fractions[,1:3])

print(irf1.SZ3$eigenvector.fractions)
print(irf2.SZ3$eigenvector.fractions)
print(irf3.SZ3$eigenvector.fractions)
print(irf4.SZ3$eigenvector.fractions)
print(irf5.SZ3$eigenvector.fractions)

save.image("MVPLN-VAR-monthly-dynamics-VAR1.RData")

q(save="no")

(save="no")

