# setup.R
#
# Sets up 1968-2008 ITERATE data for MULTIVARIATE analysis of target
# substitution effects
#
# Patrick T. Brandt
# EPPS - UT Dallas
#
# 20080425 : INITIAL VERSION
# 20081217 : Updated to include 2007 data and capture some more
#            covariates / groupings.
# 20081229 : Developed data structure / tree for the events.
#
# 20090319 : Updated extraction of the covariates so we get dummies
#            for the logistical outcomes
#
# 20090617 : Updated with 2008 data
# 20091002 : Changed the aggregation to make weekly datasets.

library(zoo)

# Read in the data to the object x
x <- read.csv("IterateCommon19682008.csv", header=TRUE, as.is=T)

# Extract out the columns we need to use for the analysis
d <- x[,c("yr", "mo", "day", "incid_no", "inc_type", "imm_vic",
          "loc_sta", "loc_end", "num_killed", "gov_killed",
          "ter_killed", "logis_suc", "nature_vic")]
rm(x)

# Rename the variables to something sensible
colnames(d) <- c("year", "month", "day", "incidentno", "incident",
                 "victim", "locationstart", "locationend", "totaldeaths",
                 "govtdeaths", "terroristdeaths", "logisticalsuccess",
                 "natureofvictim")

# Generate the subsets of the data by incident type and immediate
# victim type.

# First, recast the groupings of the victims and the incident
# types and encode as factors.

victims <- d$victim
victims[victims<3] <- 1
victims[victims>2 & victims <5] <- 2
victims[is.na(victims)==TRUE] <- 6
victims[victims==5] <- 3
victims[victims==6] <- 4
victims[victims==7] <- 4
victims[victims==8] <- 5
victims[victims==9] <- 6

vtype <- factor(victims, labels=c("Officials", "Military", "Business",
                         "Private Parties", "Terrorists",
                         "Other"))

# Now recode and factor the incident types
## 4-8, 23-25 -- armed attack & bombing
## 1,2,9,10 -- hostage
## 11 -- assassination

incidents <- d$incident
incidents[incidents==1 | incidents==2 | incidents==9 | incidents==10] <- 1
incidents[(incidents>3 & incidents<9) | (incidents>22 & incidents<26)] <- 2
incidents[incidents==11] <- 3
incidents[incidents==15 | incidents==18] <- 4


itype <- factor(incidents, labels=c("Hostage", "Bombing",
                           "Assassination", "Threats", "Sabotage",
                           "Exotic Pollution", "Nuclear Attack",
                           "Theft", "Conspiracy", "Other Actions", "Sniping",
                           "Shootout", "Arms smuggling", "Unknown"))

# Recode and factor the values for the logistical success and nature
# of the victim variables
logistical <- d$logisticalsuccess
logistical[logistical==0 | logistical==3 | logistical==4] <- 0
logistical[logistical==1 | logistical==2 | logistical==5] <- 1
logistical[logistical==6] <- 2
logistical[logistical==9] <- NA

logistical <- factor(logistical,
                     labels=c("Aborted", "Stopped", "Completed"))

natureoftarget <- d$natureofvictim
natureoftarget[natureoftarget==3] <- 1
natureoftarget <- factor(natureoftarget, labels=c("People",
                                         "Property", "Unknown"))


# Recode deaths variables to be binary indicators or whether there
# were any deaths.  This is legitimate, since the modal number of
# deaths in any incident is 1.

# First clean up the NAs
d$totaldeaths <- ifelse(is.na(d$totaldeaths)==TRUE, 0, d$totaldeaths)
d$govtdeaths <- ifelse(is.na(d$govtdeaths)==TRUE, 0, d$govtdeaths)
d$terroristdeaths <- ifelse(is.na(d$terroristdeaths)==TRUE, 0, d$terroristdeaths)


totaldeaths <- ifelse(d$totaldeaths>0, 1, 0)
govtdeaths <- ifelse(d$govtdeaths>0, 1, 0)
terroristdeaths <- ifelse(d$terroristdeaths>0, 1, 0)

# Now make a table of the incidents and victims categories
table(vtype, itype)

######################################################################
# Now subset the data by victim type and incident type.  These subsets
# will be the basis for the final time series data
######################################################################
# Steps in this part:
# 1) Subset the data by the relevant variables
# 2) Declare each subset a time series
# 3) Combine the time series together.

# Combine the aggregates incident and victim types with the original
# data.
dd <- cbind(d[,c(1:3)], vtype, itype, logistical, natureoftarget,
            totaldeaths, govtdeaths, terroristdeaths)

## ######################################################################
## # Now write out the elapsed date results for each series.
## ######################################################################
## # Main loop to do the work
## for (i in 1:nlevels(vtype))
## {
##          # subset the data
##     tmp <- subset(dd, vtype==levels(vtype)[i] &
##                   unclass(itype)<4, select=year:itype)

##          # Now deal with the cases where the combination of incident
##          # and victim types never occurs:

##     if(nrow(tmp)==0) next;

##          # Declare this to be a zoo() object and aggregate
##     tmp.dates <- as.Date(paste(tmp$year, ifelse(tmp$month==99, 6,
##                                                 tmp$month),
##                                ifelse(tmp$day==99, 15, tmp$day),
##                                sep="-"))

##     write(c(length(tmp.dates)-1, tmp.dates - as.Date("1968-1-1")),
##           file=paste(levels(vtype)[i], ".dat", sep=""), ncol=12)
##      }


rm(d)


# Now subset out just the most important types of events: hostages,
# bombings and assassinations:
ddd <- subset(dd,
              subset=(itype=="Hostage" | itype=="Bombing" | itype=="Assassination"))

## plot(seq(1:84),
## 	cumsum(as.matrix(table(deaths)/sum(ifelse(deaths>0,1,0),
## 						 na.rm=T))[2:85]),
## 	xlab="Number of Deaths",
## 	ylab="Conditional Cumulative Distribution",
## 	main="Cumulative number of deaths in ITERATE,\n conditional on one or more deaths, 1968-2007")


# Set up some dating variables.
# Note that "99" days are recoded to "15th of the month"
daily.dates <- as.Date(paste(ddd$year, ifelse(ddd$month==99, 6,
                                              ddd$month),
                             ifelse(ddd$day==99, 15, ddd$day),
                             sep="-"))

# Generate the full samples -- this is empty data series
full.daily <- zoo(, seq(as.Date("1968-01-01"), max(daily.dates),
                        by="day"))


########################################################################
# This is the data list version of the data.  Useful for the plotting
# function and specialized graphs we need, less useful for making data
# matrices for analysis and time series work.
#
########################################################################
# Main loop to do the work: this creates separate datasets for each of
# the series of target, event types, etc.  It then aggregates them to be
# daily data.
########################################################################
m <- 1             # Iterator for the list of datasets
options(warn=-1)   # turn off the nasty zoo warnings, since they do
                   # not really matter for this application
########################################################################
# Storage for the objects we create: need to work this so it
# generates a smartly constructed list.
########################################################################

x <- list()

for (i in 1:nlevels(ddd$vtype))  # victim type
{
    for(k in 1:nlevels(ddd$logistical)) # logistical success
    {
        for(l in 1:nlevels(ddd$natureoftarget)) # nature of victim entity
        {

                # subset the data
            tmp <- subset(ddd,
                          vtype==levels(vtype)[i] &
                          logistical==levels(logistical)[k] &
                          natureoftarget==levels(natureoftarget)[l])

         # Now deal with the cases where the combination of incident
         # logistical, and victim types never occurs:

            cat("May have built dataset ", m, "\n")

            if(nrow(tmp)==0) next;


         # add an indicator for the event
            tmp <- cbind(tmp, rep(1, nrow(tmp)))

         # Declare this to be a zoo() object and aggregate
            tmp.dates <- as.Date(paste(tmp$year,
                                       ifelse(tmp$month==99, 6, tmp$month),
                                       ifelse(tmp$day==99, 15, tmp$day),
                                       sep="-"))

            tmp2 <- aggregate(zoo(tmp[,8:11], tmp.dates),
                              tmp.dates,
                              sum)

          # label the columns
            colnames(tmp2) <- c(colnames(tmp[,8:10]), "Events")

          # Now place all of this into a series of lists.
            x[[m]] <- list(vtype=levels(vtype)[i],
                           logistical=levels(logistical)[k],
                           natureoftarget=levels(natureoftarget)[l],
                           data=tmp2)

          # Iterate and continue
            m <- m+1

        }
    }
}

rm(tmp, tmp2, tmp.dates)

options(warn=0)
########################################################################
# This finishes building the aggregated daily datasets LIST.
########################################################################

# Now save the list of datasets and the indexing for the full daily
# time series.  We can then use these to construct and plot the
# relevant time series graphics.
save(x, full.daily, file="TargetSorted1968-2008.RData")

########################################################################

########################################################################
# NOW BUILD THE SQUARE ARRAY VERSION FOR EACH OF THE DATA SERIES.  DO
# THIS DAILY TO START, SINCE THEN WE CAN AGGREGATE FURTHER.
########################################################################
# Now make the simple array.  To do this, we
# first need to expand out some factors to dummies.
########################################################################

ddd$logistical <- ifelse(is.na(ddd$logistical)==TRUE, 0,
                         ddd$logistical)
ddd$natureoftarget <- ifelse(is.na(ddd$natureoftarget)==TRUE, 0,
                         ddd$natureoftarget)


aborted <- ifelse(ddd$logistical==1, 1, 0)
stopped <- ifelse(ddd$logistical==2, 1, 0)
completed <- ifelse(ddd$logistical==3, 1, 0)

people <- ifelse(ddd$natureoftarget==1, 1, 0)
property <- ifelse(ddd$natureoftarget==2, 1, 0)
unknown <- ifelse(ddd$natureoftarget==3, 1, 0)

dddd <- cbind(ddd, aborted, stopped, completed,
              people, property, unknown)

# Now we need to aggregate this by dates.  So define the dates and the
# various subsets.

for(i in 1:nlevels(dddd$vtype))
{
    # subset
    tmp <- subset(dddd, vtype==levels(vtype)[i])

    # dates
    tmp.dates <- as.Date(paste(tmp$year,
                               ifelse(tmp$month==99, 6, tmp$month),
                               ifelse(tmp$day==99, 15, tmp$day),
                               sep="-"))

    # event measure and dates
    tmp <- cbind(tmp.dates, tmp, rep(1, nrow(tmp)))

    # Aggregate daily, declare as zoo.
    tmp2 <- aggregate(tmp[,9:18], by=list(tmp.dates), sum)

    tmp2 <- zoo(tmp2[,2:11], tmp2[,1])
    tmp2 <- merge(tmp2, full.daily, fill=0)
    colnames(tmp2) <- c(colnames(tmp2[,1:9]), "event")

##     # Aggregate to monthly
##     tmp2 <- aggregate(tmp2, as.yearmon, sum)

    # Aggregate to weekly

    week <- as.numeric(format(index(tmp2), "%U"))
    year <- as.numeric(format(index(tmp2), "%Y"))
    year.week <- paste(year, week, sep=".")
    tmp2 <- ts(aggregate(as.matrix(tmp2), by=list(year.week), sum),
                         start=c(1968,1), freq=52)

    assign(sub(" ", "", levels(vtype)[i], perl=T), tmp2)

}

## # Now compile the monthly datasets into a list
## x.m <- list(Officials=Officials,
##             Military=Military,
##             Business=Business,
##             PrivateParties=PrivateParties)

# Now compile the weekly datasets into a list
x.w <- list(Officials=Officials,
            Military=Military,
            Business=Business,
            PrivateParties=PrivateParties)

#  save(x.m, file="MonthlySeries1968-2008.RData")

save(x.w, file="WeeklySeries1968-2008.RData")


# Exit gracefully
q(save="no")
